@@ -1,11 +1,12 @@
 """Provide a model for the Z-Wave JS node."""
+
 from __future__ import annotations
 
 import asyncio
 import copy
-import logging
 from datetime import datetime
-from typing import TYPE_CHECKING, Any, cast
+import logging
+from typing import TYPE_CHECKING, Any, Literal, cast
 
 from ...const import (
     INTERVIEW_FAILED,
@@ -14,19 +15,15 @@ from ...const import (
     DateAndTime,
     NodeStatus,
     PowerLevel,
+    Protocols,
     SecurityClass,
 )
 from ...event import Event, EventBase
-from ...exceptions import (
-    FailedCommand,
-    NotFoundError,
-    UnparseableValue,
-    UnwriteableValue,
-)
+from ...exceptions import NotFoundError, UnparseableValue, UnwriteableValue
 from ..command_class import CommandClassInfo
 from ..device_class import DeviceClass
 from ..device_config import DeviceConfig
-from ..endpoint import Endpoint
+from ..endpoint import Endpoint, EndpointDataType
 from ..notification import (
     EntryControlNotification,
     EntryControlNotificationDataType,
@@ -39,14 +36,15 @@ from ..notification import (
 )
 from ..value import (
     ConfigurationValue,
+    ConfigurationValueFormat,
     MetaDataType,
+    SetConfigParameterResult,
     SetValueResult,
     Value,
     ValueDataType,
     ValueMetadata,
     ValueNotification,
     _get_value_id_str_from_dict,
-    _init_value,
 )
 from .data_model import NodeDataType
 from .event_model import NODE_EVENT_MODEL_MAP
@@ -102,7 +100,7 @@ def _get_value_id_dict_from_value_data(value_data: ValueDataType) -> dict[str, A
 class Node(EventBase):
     """Represent a Z-Wave JS node."""
 
-    def __init__(self, client: "Client", data: NodeDataType) -> None:
+    def __init__(self, client: Client, data: NodeDataType) -> None:
         """Initialize the node."""
         super().__init__()
         self.client = client
@@ -112,9 +110,11 @@ class Node(EventBase):
             client, data.get("statistics", DEFAULT_NODE_STATISTICS)
         )
         self._firmware_update_progress: NodeFirmwareUpdateProgress | None = None
+        self._device_class: DeviceClass | None = None
+        self._last_seen: datetime | None = None
         self.values: dict[str, ConfigurationValue | Value] = {}
         self.endpoints: dict[int, Endpoint] = {}
-        self._status_event = asyncio.Event()
+        self.status_event = asyncio.Event()
         self.update(data)
 
     def __repr__(self) -> str:
@@ -133,6 +133,12 @@ class Node(EventBase):
             self.client.driver == other.client.driver and self.node_id == other.node_id
         )
 
+    def _init_value(self, val: ValueDataType) -> Value | ConfigurationValue:
+        """Initialize a Value object from ValueDataType."""
+        if val["commandClass"] == CommandClass.CONFIGURATION:
+            return ConfigurationValue(self, val)
+        return Value(self, val)
+
     @property
     def node_id(self) -> int:
         """Return node ID property."""
@@ -146,9 +152,7 @@ class Node(EventBase):
     @property
     def device_class(self) -> DeviceClass | None:
         """Return the device_class."""
-        if (device_class := self.data.get("deviceClass")) is None:
-            return None
-        return DeviceClass(device_class)
+        return self._device_class
 
     @property
     def installer_icon(self) -> int | None:
@@ -356,9 +360,7 @@ class Node(EventBase):
     @property
     def last_seen(self) -> datetime | None:
         """Return when the node was last seen."""
-        if last_seen := self.data.get("lastSeen"):
-            return datetime.fromisoformat(last_seen)
-        return None
+        return self._last_seen
 
     @property
     def default_volume(self) -> int | float | None:
@@ -370,17 +372,42 @@ class Node(EventBase):
         """Return the default transition duration."""
         return self.data.get("defaultTransitionDuration")
 
-    def update(self, data: NodeDataType) -> None:
-        """Update the internal state data."""
-        self.data = copy.deepcopy(data)
-        self._device_config = DeviceConfig(self.data.get("deviceConfig", {}))
-        self._statistics = NodeStatistics(
-            self.client, self.data.get("statistics", DEFAULT_NODE_STATISTICS)
-        )
+    @property
+    def protocol(self) -> Protocols | None:
+        """Return the protocol used to communicate with this node."""
+        if "protocol" in self.data:
+            return Protocols(self.data["protocol"])
+        return None
 
+    def _update_endpoints(self, endpoints: list[EndpointDataType]) -> None:
+        """Update the endpoints data."""
+        new_endpoints_data = {endpoint["index"]: endpoint for endpoint in endpoints}
+        new_endpoint_idxs = set(new_endpoints_data)
+        stale_endpoint_idxs = set(self.endpoints) - new_endpoint_idxs
+
+        # Remove stale endpoints
+        for endpoint_idx in stale_endpoint_idxs:
+            self.endpoints.pop(endpoint_idx)
+
+        # Add new endpoints or update existing ones
+        for endpoint_idx in new_endpoint_idxs:
+            endpoint = new_endpoints_data[endpoint_idx]
+            values = {
+                value_id: value
+                for value_id, value in self.values.items()
+                if self.index == value.endpoint
+            }
+            if endpoint_idx in self.endpoints:
+                self.endpoints[endpoint_idx].update(endpoint, values)
+            else:
+                self.endpoints[endpoint_idx] = Endpoint(
+                    self.client, self, endpoint, values
+                )
+
+    def _update_values(self, values: list[ValueDataType]) -> None:
+        """Update the values data."""
         new_values_data = {
-            _get_value_id_str_from_dict(self, val): val
-            for val in self.data.pop("values")
+            _get_value_id_str_from_dict(self, val): val for val in values
         }
         new_value_ids = set(new_values_data)
         stale_value_ids = set(self.values) - new_value_ids
@@ -402,33 +429,31 @@ class Node(EventBase):
                 if value_id in self.values:
                     self.values[value_id].update(val)
                 else:
-                    self.values[value_id] = _init_value(self, val)
+                    self.values[value_id] = self._init_value(val)
             except UnparseableValue:
                 # If we can't parse the value, don't store it
                 pass
 
-        new_endpoints_data = {
-            endpoint["index"]: endpoint for endpoint in self.data.pop("endpoints")
-        }
-        new_endpoint_idxs = set(new_endpoints_data)
-        stale_endpoint_idxs = set(self.endpoints) - new_endpoint_idxs
+    def update(self, data: NodeDataType) -> None:
+        """Update the internal state data."""
+        self.data = copy.deepcopy(data)
+        self._device_config = DeviceConfig(self.data.get("deviceConfig", {}))
+        if (device_class := self.data.get("deviceClass")) is None:
+            self._device_class = None
+        else:
+            self._device_class = DeviceClass(device_class)
 
-        # Remove stale endpoints
-        for endpoint_idx in stale_endpoint_idxs:
-            self.endpoints.pop(endpoint_idx)
+        self._statistics = NodeStatistics(
+            self.client, self.data.get("statistics", DEFAULT_NODE_STATISTICS)
+        )
+        if last_seen := data.get("lastSeen"):
+            self._last_seen = datetime.fromisoformat(last_seen)
+        if not self._statistics.last_seen and self.last_seen:
+            self._statistics.last_seen = self.last_seen
+            self._statistics.data["lastSeen"] = self.last_seen.isoformat()
 
-        # Add new endpoints or update existing ones
-        for endpoint_idx in new_endpoint_idxs - stale_endpoint_idxs:
-            endpoint = new_endpoints_data[endpoint_idx]
-            values = {
-                value_id: value
-                for value_id, value in self.values.items()
-                if self.index == value.endpoint
-            }
-            if endpoint_idx in self.endpoints:
-                self.endpoints[endpoint_idx].update(endpoint, values)
-            else:
-                self.endpoints[endpoint_idx] = Endpoint(self.client, endpoint, values)
+        self._update_values(self.data.pop("values"))
+        self._update_endpoints(self.data.pop("endpoints"))
 
     def get_command_class_values(
         self, command_class: CommandClass, endpoint: int | None = None
@@ -450,7 +475,7 @@ class Node(EventBase):
 
     def receive_event(self, event: Event) -> None:
         """Receive an event."""
-        NODE_EVENT_MODEL_MAP[event.type](**event.data)
+        NODE_EVENT_MODEL_MAP[event.type].from_dict(event.data)
 
         self._handle_event_protocol(event)
         event.data["node"] = self
@@ -467,8 +492,8 @@ class Node(EventBase):
         """
         Send a node command. For internal use only.
 
-        If wait_for_result is not None, it will take precedence, otherwise we will decide
-        to wait or not based on the node status.
+        If wait_for_result is not None, it will take precedence, otherwise we will
+        decide to wait or not based on the node status.
         """
         kwargs = {}
         message = {"command": f"node.{cmd}", "nodeId": self.node_id, **cmd_kwargs}
@@ -485,13 +510,13 @@ class Node(EventBase):
             result_task = asyncio.create_task(
                 self.client.async_send_command(message, **kwargs)
             )
-            status_task = asyncio.create_task(self._status_event.wait())
+            status_task = asyncio.create_task(self.status_event.wait())
             await asyncio.wait(
                 [result_task, status_task],
                 return_when=asyncio.FIRST_COMPLETED,
             )
             status_task.cancel()
-            if self._status_event.is_set() and not result_task.done():
+            if self.status_event.is_set() and not result_task.done():
                 result_task.cancel()
                 return None
             return result_task.result()
@@ -569,12 +594,9 @@ class Node(EventBase):
         data = await self.async_send_command(
             "get_defined_value_ids", wait_for_result=True
         )
-
-        if data is None:
-            # We should never reach this code
-            raise FailedCommand("Command failed", "failed_command")
+        assert data
         return [
-            _init_value(self, cast(ValueDataType, value_id))
+            self._init_value(cast(ValueDataType, value_id))
             for value_id in data["valueIds"]
         ]
 
@@ -704,7 +726,7 @@ class Node(EventBase):
         return None
 
     async def async_test_power_level(
-        self, test_node: "Node", power_level: PowerLevel, test_frame_count: int
+        self, test_node: Node, power_level: PowerLevel, test_frame_count: int
     ) -> int:
         """Send testPowerLevel command to Node."""
         data = await self.async_send_command(
@@ -735,7 +757,7 @@ class Node(EventBase):
         return LifelineHealthCheckSummary(data["summary"])
 
     async def async_check_route_health(
-        self, target_node: "Node", rounds: int | None = None
+        self, target_node: Node, rounds: int | None = None
     ) -> RouteHealthCheckSummary:
         """Send checkRouteHealth command to Node."""
         kwargs = {"targetNodeId": target_node.node_id}
@@ -840,7 +862,7 @@ class Node(EventBase):
         return cast(int, data["timestamp"])
 
     async def async_manually_idle_notification_value(self, val: Value | str) -> None:
-        """Send manuallyIdleNotificationValue command to Node for given value (or value_id)."""
+        """Send manuallyIdleNotificationValue cmd to Node for value (or value_id)."""
         # a value may be specified as value_id or the value itself
         if not isinstance(val, Value):
             val = self.values[val]
@@ -941,6 +963,19 @@ class Node(EventBase):
             return cast(bool, changed)
         return None
 
+    async def async_set_raw_config_parameter_value(
+        self,
+        new_value: int | str,
+        property_: int | str,
+        property_key: int | None = None,
+        value_size: Literal[1, 2, 4] | None = None,
+        value_format: ConfigurationValueFormat | None = None,
+    ) -> tuple[Value, SetConfigParameterResult | None]:
+        """Send setRawConfigParameterValue."""
+        return await self.endpoints[0].async_set_raw_config_parameter_value(
+            new_value, property_, property_key, value_size, value_format
+        )
+
     def handle_test_powerlevel_progress(self, event: Event) -> None:
         """Process a test power level progress event."""
         event.data["test_power_level_progress"] = TestPowerLevelProgress(
@@ -968,25 +1003,25 @@ class Node(EventBase):
     def handle_wake_up(self, event: Event) -> None:
         """Process a node wake up event."""
         # pylint: disable=unused-argument
-        self._status_event.clear()
+        self.status_event.clear()
         self.data["status"] = NodeStatus.AWAKE
 
     def handle_sleep(self, event: Event) -> None:
         """Process a node sleep event."""
         # pylint: disable=unused-argument
-        self._status_event.set()
+        self.status_event.set()
         self.data["status"] = NodeStatus.ASLEEP
 
     def handle_dead(self, event: Event) -> None:
         """Process a node dead event."""
         # pylint: disable=unused-argument
-        self._status_event.set()
+        self.status_event.set()
         self.data["status"] = NodeStatus.DEAD
 
     def handle_alive(self, event: Event) -> None:
         """Process a node alive event."""
         # pylint: disable=unused-argument
-        self._status_event.clear()
+        self.status_event.clear()
         self.data["status"] = NodeStatus.ALIVE
 
     def handle_interview_started(self, event: Event) -> None:
@@ -1024,7 +1059,7 @@ class Node(EventBase):
         value_id = _get_value_id_str_from_dict(self, evt_val_data)
         value = self.values.get(value_id)
         if value is None:
-            value = _init_value(self, evt_val_data)
+            value = self._init_value(evt_val_data)
             self.values[value.value_id] = event.data["value"] = value
         else:
             value.receive_event(event)
@@ -1082,10 +1117,10 @@ class Node(EventBase):
 
     def handle_firmware_update_progress(self, event: Event) -> None:
         """Process a node firmware update progress event."""
-        self._firmware_update_progress = event.data[
-            "firmware_update_progress"
-        ] = NodeFirmwareUpdateProgress(
-            self, cast(NodeFirmwareUpdateProgressDataType, event.data["progress"])
+        self._firmware_update_progress = event.data["firmware_update_progress"] = (
+            NodeFirmwareUpdateProgress(
+                self, cast(NodeFirmwareUpdateProgressDataType, event.data["progress"])
+            )
         )
 
     def handle_firmware_update_finished(self, event: Event) -> None:
@@ -1101,5 +1136,5 @@ class Node(EventBase):
         event.data["statistics_updated"] = self._statistics = NodeStatistics(
             self.client, statistics
         )
-        if last_seen := statistics.get("lastSeen"):
-            self.data["lastSeen"] = last_seen
+        if self._statistics.last_seen:
+            self._last_seen = self._statistics.last_seen
